
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/layers.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <stdlib.h>

struct Region *install_region (struct Window *win)

{
struct Region *region = NewRegion();
struct Rectangle rect;
rect.MinX = 20;
rect.MinY = 20;
rect.MaxX = win->GZZWidth  - 21;
rect.MaxY = win->GZZHeight - 21;
OrRectRegion (region,&rect);
rect.MinX = win->GZZWidth  / 2 - 25;
rect.MinY = win->GZZHeight / 2 - 25;
rect.MaxX = win->GZZWidth  / 2 + 25;
rect.MaxY = win->GZZHeight / 2 + 25;
XorRectRegion (region,&rect);
return (InstallClipRegion (win->RPort->Layer,region));
}


void remove_region (struct Window *win,struct Region *oldreg)

{
struct Region *region = InstallClipRegion (win->RPort->Layer,oldreg);
DisposeRegion (region);
}


void draw_line (struct Window *win)

{
static long x1 = 100;
static long y1 = 100;
static long x2 = 100;
static long y2 = 100;
static long vx1 = -4;
static long vy1 = 1;
static long vx2 = 2;
static long vy2 = -3;
static long pen = 0;

x1 += vx1;
if (x1 < 0)
	{
	x1 = 0;
	vx1 = -vx1;
	}
else if (x1 >= win->GZZWidth)
	{
	x1 = win->GZZWidth - 1;
	vx1 = -vx1;
	}
y1 += vy1;
if (y1 < 0)
	{
	y1 = 0;
	vy1 = -vy1;
	}
else if (y1 >= win->GZZHeight)
	{
	y1 = win->GZZHeight - 1;
	vy1 = -vy1;
	}
x2 += vx2;
if (x2 < 0)
	{
	x2 = 0;
	vx2 = -vx2;
	}
else if (x2 >= win->GZZWidth)
	{
	x2 = win->GZZWidth - 1;
	vx2 = -vx2;
	}
y2 += vy2;
if (y2 < 0)
	{
	y2 = 0;
	vy2 = -vy2;
	}
else if (y2 >= win->GZZHeight)
	{
	y2 = win->GZZHeight - 1;
	vy2 = -vy2;
	}
pen ++;
SetAPen (win->RPort,pen);
Move (win->RPort,x1,y1);
Draw (win->RPort,x2,y2);
}

int main (void)

{
struct RDArgs *rdargs;
struct {
	long on;
	} args = {0};
struct Window *win;
struct IntuiMessage *imsg;
BOOL cont;
struct Region *oldreg;

rdargs = ReadArgs ("ON/S",(APTR)&args,NULL);
if (!rdargs)
	{
	PrintFault (IoErr(),NULL);
	return (RETURN_ERROR);
	}

if (win = OpenWindowTags (NULL,
		WA_Width,200,
		WA_Height,200,
		WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_SIZEGADGET|WFLG_ACTIVATE|WFLG_GIMMEZEROZERO|WFLG_NOCAREREFRESH,
		WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_VANILLAKEY|IDCMP_NEWSIZE,
		WA_MinWidth,100,
		WA_MinHeight,100,
		WA_MaxWidth,0x7fff,
		WA_MaxHeight,0x7fff,
		WA_Title,"Region Test",
		TAG_END))
	{

	SetAPen (win->RPort,0);
	RectFill (win->RPort,0,0,win->GZZWidth-1,win->GZZHeight-2);
	oldreg = install_region (win);
	SetAPen (win->RPort,1);
	RectFill (win->RPort,0,0,win->GZZWidth-1,win->GZZHeight-2);

	cont = TRUE;
	do	{
		WaitTOF();
		draw_line (win);

		if (SetSignal (0,0) & SIGBREAKF_CTRL_C)
			cont = FALSE;

		while (imsg = (struct IntuiMessage *) GetMsg (win->UserPort))
			{
			switch (imsg->Class)
				{
			case IDCMP_NEWSIZE:
				remove_region (win,oldreg);
				SetAPen (win->RPort,0);
				RectFill (win->RPort,0,0,win->GZZWidth-1,win->GZZHeight-2);
				oldreg = install_region (win);
				SetAPen (win->RPort,1);
				RectFill (win->RPort,0,0,win->GZZWidth-1,win->GZZHeight-2);
				break;
			case IDCMP_VANILLAKEY:
				if (imsg->Code == 0x1b)
					cont = FALSE;
				break;
			case IDCMP_CLOSEWINDOW:
				cont = FALSE;
				break;
				}
			ReplyMsg ((struct Message *)imsg);
			}
		}
	while (cont);

	remove_region (win,oldreg);

	CloseWindow (win);
	}

FreeArgs (rdargs);

return (RETURN_OK);
}
